@@ -4,19 +4,14 @@
 --       AUTHOR:  Khaled Hosny, Élie Roux, Philipp Gesang
 -----------------------------------------------------------------------
 
-local ProvidesLuaModule = { 
+assert(luaotfload_module, "This is a part of luaotfload and should not be loaded independently") { 
     name          = "luaotfload-auxiliary",
-    version       = "2.97",       --TAGVERSION
-    date          = "2019-05-18", --TAGDATE
+    version       = "3.16",       --TAGVERSION
+    date          = "2020-12-31", --TAGDATE
     description   = "luaotfload submodule / auxiliary functions",
     license       = "GPL v2.0"
 }
 
-if luatexbase and luatexbase.provides_module then
-  luatexbase.provides_module (ProvidesLuaModule)
-end  
-
-
 luaotfload                  = luaotfload or { }
 local log                   = luaotfload.log
 local logreport             = log.report
@@ -31,14 +26,22 @@ local texsprint             = tex.sprint
 local dofile                = dofile
 local getmetatable          = getmetatable
 local setmetatable          = setmetatable
-local utf8                  = unicode.utf8
+local utfcodepoint          = utf8.codepoint
 local stringlower           = string.lower
+local stringupper           = string.upper
 local stringformat          = string.format
 local stringgsub            = string.gsub
 local stringbyte            = string.byte
 local stringfind            = string.find
 local tablecopy             = table.copy
 
+local harf = luaotfload.harfbuzz
+local GSUBtag, GPOStag
+if harf then
+  GSUBtag = harf.Tag.new("GSUB")
+  GPOStag = harf.Tag.new("GPOS")
+end
+
 local aux                   = { }
 local luaotfload_callbacks  = { }
 
@@ -48,7 +51,7 @@ local luaotfload_callbacks  = { }
 
 --- https://github.com/khaledhosny/luaotfload/issues/54
 
-local rewrite_fontname = function (tfmdata, specification)
+local function rewrite_fontname(tfmdata, specification)
   local format = tfmdata.format or tfmdata.properties.format
   if stringfind (specification, " ") then
     tfmdata.name = stringformat ("%q", specification)
@@ -60,7 +63,7 @@ end
 
 local rewriting = false
 
-local start_rewrite_fontname = function ()
+function aux.start_rewrite_fontname()
   if rewriting == false then
     luatexbase.add_to_callback (
       "luaotfload.patch_font",
@@ -72,9 +75,7 @@ local start_rewrite_fontname = function ()
   end
 end
 
-aux.start_rewrite_fontname = start_rewrite_fontname
-
-local stop_rewrite_fontname = function ()
+function aux.stop_rewrite_fontname()
   if rewriting == true then
     luatexbase.remove_from_callback
       ("luaotfload.patch_font", "luaotfload.rewrite_fontname")
@@ -84,15 +85,13 @@ local stop_rewrite_fontname = function ()
   end
 end
 
-aux.stop_rewrite_fontname = stop_rewrite_fontname
-
 
 --[[doc--
 This sets two dimensions apparently relied upon by the unicode-math
 package.
 --doc]]--
 
-local set_sscale_dimens = function (fontdata)
+local function set_sscale_dimens(fontdata)
   local resources     = fontdata.resources      if not resources     then return end
   local mathconstants = resources.MathConstants if not mathconstants then return end
   local parameters    = fontdata.parameters     if not parameters    then return end
@@ -105,10 +104,18 @@ luaotfload_callbacks [#luaotfload_callbacks + 1] = {
   "patch_font", set_sscale_dimens, "set_sscale_dimens",
 }
 
+-- Starting with LuaTeX 1.11.2, this is a more reliable way of selecting the
+-- right font than relying on psnames
+luaotfload_callbacks [#luaotfload_callbacks + 1] = {
+  "patch_font", function(fontdata)
+    fontdata.subfont = fontdata.specification.sub or 1
+  end, "set_font_index",
+}
+
 local default_units = 1000
 
 --- fontobj -> int
-local lookup_units = function (fontdata)
+local function lookup_units(fontdata)
   local units = fontdata.units
   if units and units > 0 then return units end
   local shared     = fontdata.shared    if not shared     then return default_units end
@@ -126,7 +133,7 @@ end
 This callback corrects some values of the Cambria font.
 --doc]]--
 --- fontobj -> unit
-local patch_cambria_domh = function (fontdata)
+local function patch_cambria_domh(fontdata)
   local mathconstants = fontdata.MathConstants
   if mathconstants and fontdata.psname == "CambriaMath" then
     --- my test Cambria has 2048
@@ -155,15 +162,13 @@ luaotfload_callbacks [#luaotfload_callbacks + 1] = {
 --doc]]--
 
 --- fontobj -> unit
-local fixup_fontdata = function (data)
-
+local function fixup_fontdata(data)
   local t = type (data)
   --- Some OT fonts like Libertine R lack the resources table, causing
   --- the fontloader to nil-index.
   if t == "table" then
     if data and not data.resources then data.resources = { } end
   end
-
 end
 
 luaotfload_callbacks [#luaotfload_callbacks + 1] = {
@@ -187,15 +192,14 @@ Comment from fontspec:
 
 local capheight_reference_chars      = { "X", "M", "Ж", "ξ", }
 local capheight_reference_codepoints do
-  local utfbyte = unicode.utf8.byte
   capheight_reference_codepoints = { }
   for i = 1, #capheight_reference_chars do
     local chr = capheight_reference_chars [i]
-    capheight_reference_codepoints [i] = utfbyte (chr)
+    capheight_reference_codepoints [i] = utfcodepoint (chr)
   end
 end
 
-local determine_capheight = function (fontdata)
+local function determine_capheight(fontdata)
   local parameters = fontdata.parameters if not parameters then return false end
   local characters = fontdata.characters if not characters then return false end
   --- Pretty simplistic but it does return *some* value for most fonts;
@@ -206,7 +210,7 @@ local determine_capheight = function (fontdata)
     local refchar = characters [refcp]
     if refchar then
       logreport ("both", 4, "aux",
-                 "picked height of character ‘%s’ (U+%d) as \\fontdimen8 \z
+                 "picked height of character '%s' (U+%04X) as \\fontdimen8 \z
                   candidate",
                  capheight_reference_chars [i], refcp)
       return refchar.height
@@ -215,7 +219,7 @@ local determine_capheight = function (fontdata)
   return false
 end
 
-local query_ascender = function (fontdata)
+local function query_ascender(fontdata)
   local parameters = fontdata.parameters if not parameters then return false end
   local ascender   = parameters.ascender
   if ascender then
@@ -232,7 +236,7 @@ local query_ascender = function (fontdata)
   return ascender * size / units --- scaled
 end
 
-local query_capheight = function (fontdata)
+local function query_capheight(fontdata)
   local parameters = fontdata.parameters  if not parameters then return false end
   local shared     = fontdata.shared      if not shared     then return false end
   local rawdata    = shared.rawdata       if not rawdata    then return false end
@@ -244,20 +248,20 @@ local query_capheight = function (fontdata)
   return capheight * size / units
 end
 
-local query_fontdimen8 = function (fontdata)
+local function query_fontdimen8(fontdata)
   local parameters = fontdata.parameters if not parameters then return false end
   local fontdimen8 = parameters [8]
   if fontdimen8 then return fontdimen8 end
   return false
 end
 
-local caphtfmt = function (ref, ht)
+local function caphtfmt(ref, ht)
   if not ht  then return "<none>"      end
   if not ref then return tostring (ht) end
   return stringformat ("%s(δ=%s)", ht, ht - ref)
 end
 
-local set_capheight = function (fontdata)
+local function set_capheight(fontdata)
     if not fontdata then
       logreport ("both", 0, "aux",
                  "error: set_capheight() received garbage")
@@ -287,12 +291,27 @@ luaotfload_callbacks [#luaotfload_callbacks + 1] = {
   "patch_font", set_capheight, "set_capheight",
 }
 
+-- Of course there are also fonts with no sensible x-height, so let's add a
+-- fallback there:
+local function set_xheight(tfmdata)
+  local parameters = tfmdata.parameters
+  if not parameters then return end
+  if not ((parameters.x_height or parameters[5] or 0) == 0) then return end
+  if tfmdata.characters and tfmdata.characters[120] then
+    parameters.x_height = tfmdata.characters[120].height
+  else
+    parameters.x_height = (parameters.ascender or 0)/2
+  end
+end
+luaotfload_callbacks [#luaotfload_callbacks + 1] = {
+  "patch_font", set_xheight, "set_xheight",
+}
 -----------------------------------------------------------------------
 ---                      glyphs and characters
 -----------------------------------------------------------------------
 
 --- int -> int -> bool
-local font_has_glyph = function (font_id, codepoint)
+function aux.font_has_glyph(font_id, codepoint)
   local fontdata = fonts.hashes.identifiers[font_id]
   if fontdata then
     if fontdata.characters[codepoint] ~= nil then return true end
@@ -300,11 +319,9 @@ local font_has_glyph = function (font_id, codepoint)
   return false
 end
 
-aux.font_has_glyph = font_has_glyph
-
 --- undocumented
 
-local raw_slot_of_name = function (font_id, glyphname)
+local function raw_slot_of_name(font_id, glyphname)
   local fontdata = font.fonts[font_id]
   if fontdata.type == "virtual" then --- get base font for glyph idx
     local codepoint  = encodings.agl.unicodes[glyphname]
@@ -331,7 +348,7 @@ end
 --doc]]--
 
 --- int -> string -> bool -> (int | false)
-local slot_of_name = function (font_id, glyphname, unsafe)
+function aux.slot_of_name(font_id, glyphname, unsafe)
   if not font_id   or type (font_id)   ~= "number"
   or not glyphname or type (glyphname) ~= "string"
   then
@@ -343,22 +360,32 @@ local slot_of_name = function (font_id, glyphname, unsafe)
 
   local tfmdata = identifiers [font_id]
   if not tfmdata then return raw_slot_of_name (font_id, glyphname) end
-  local resources = tfmdata.resources  if not resources then return false end
-  local unicodes  = resources.unicodes if not unicodes  then return false end
+  local hbdata = tfmdata.hb
+  if hbdata then
+    local hbshared = hbdata.shared
+    local nominals = hbshared.nominals
+    local hbfont = hbshared.font
 
-  local unicode = unicodes [glyphname]
-  if unicode then
-    if type (unicode) == "number" then
-      return unicode
-    else
-      return unicode [1] --- for multiple components
+    local gid = hbfont:get_glyph_from_name(glyphname)
+    if gid ~= nil then
+      return nominals[gid] or gid + hbshared.gid_offset
+    end
+  else
+    local resources = tfmdata.resources  if not resources then return false end
+    local unicodes  = resources.unicodes if not unicodes  then return false end
+
+    local unicode = unicodes [glyphname]
+    if unicode then
+      if type (unicode) == "number" then
+        return unicode
+      else
+        return unicode [1] --- for multiple components
+      end
     end
   end
   return false
 end
 
-aux.slot_of_name = slot_of_name
-
 --[[doc--
 
   Inverse of above; not authoritative as to my knowledge the official
@@ -373,7 +400,7 @@ aux.slot_of_name = slot_of_name
 local indices
 
 --- int -> (string | false)
-local name_of_slot = function (codepoint)
+function aux.name_of_slot(codepoint)
   if not codepoint or type (codepoint) ~= "number" then
     logreport ("both", 0, "aux",
                "invalid parameters to name_of_slot (%s)",
@@ -397,20 +424,35 @@ local name_of_slot = function (codepoint)
   return false
 end
 
-aux.name_of_slot      = name_of_slot
+--[[doc--
 
+  Get the GID of the glyph associated with a given name.
+
+--doc]]--
+function aux.gid_of_name(font_id, glyphname)
+  local slot = aux.slot_of_name(font_id, glyphname)
+  if not slot then return end
+  local tfmdata = identifiers[font_id] or font.fonts[font_id]
+  -- assert(tfmdata) -- Otherwise slot_of_name would have failed already
+  return tfmdata.characters[slot].index or slot
+end
 
 -----------------------------------------------------------------------
 ---                 features / scripts / languages
 -----------------------------------------------------------------------
 --- lots of arrowcode ahead
 
-local get_features = function (tfmdata)
+local function get_features(tfmdata)
   local resources = tfmdata.resources  if not resources then return false end
   local features  = resources.features if not features  then return false end
   return features
 end
 
+local function get_hbface(tfmdata)
+  if not tfmdata.hb then return end
+  return tfmdata.hb.shared.face
+end
+
 --[[doc--
 This function, modeled after “check_script()” from fontspec, returns
 true if in the given font, the script “asked_script” is accounted for in at
@@ -418,7 +460,7 @@ least one feature.
 --doc]]--
 
 --- int -> string -> bool
-local provides_script = function (font_id, asked_script)
+function aux.provides_script(font_id, asked_script)
   if not font_id      or type (font_id)      ~= "number"
   or not asked_script or type (asked_script) ~= "string"
   then
@@ -428,9 +470,22 @@ local provides_script = function (font_id, asked_script)
     return false
   end
   asked_script = stringlower(asked_script)
-  if font_id and font_id > 0 then
-    local tfmdata = identifiers[font_id]
-    if not tfmdata then return false end
+  local tfmdata = identifiers[font_id]
+  if not tfmdata then
+    logreport ("log", 0, "aux", "no font with id %d", font_id)
+    return false
+  end
+  local hbface = get_hbface(tfmdata)
+  if hbface then
+    local script = harf.Tag.new(asked_script == "dflt" and "DFLT"
+                                                        or asked_script)
+    for _, tag in next, { GSUBtag, GPOStag } do
+      if hbface:ot_layout_find_script(tag, script) then
+        return true
+      end
+    end
+    return false
+  else
     local features = get_features (tfmdata)
     if features == false then
       logreport ("log", 1, "aux", "font no %d lacks a features table", font_id)
@@ -451,12 +506,9 @@ local provides_script = function (font_id, asked_script)
                "font no %d (%s) defines no feature for script %s",
                font_id, fontname, asked_script)
   end
-  logreport ("log", 0, "aux", "no font with id %d", font_id)
   return false
 end
 
-aux.provides_script = provides_script
-
 --[[doc--
 This function, modeled after “check_language()” from fontspec, returns
 true if in the given font, the language with tage “asked_language” is
@@ -465,7 +517,7 @@ feature.
 --doc]]--
 
 --- int -> string -> string -> bool
-local provides_language = function (font_id, asked_script, asked_language)
+function aux.provides_language(font_id, asked_script, asked_language)
   if not font_id        or type (font_id)        ~= "number"
   or not asked_script   or type (asked_script)   ~= "string"
   or not asked_language or type (asked_language) ~= "string"
@@ -478,8 +530,30 @@ local provides_language = function (font_id, asked_script, asked_language)
     return false
   end
   asked_script   = stringlower(asked_script)
-  asked_language = stringlower(asked_language)
-  if font_id and font_id > 0 then
+  local tfmdata = identifiers[font_id]
+  if not tfmdata then
+    logreport ("log", 0, "aux", "no font with id %d", font_id)
+    return false
+  end
+  local hbface = get_hbface(tfmdata)
+  if hbface then
+    asked_language  = stringupper(asked_language)
+    if asked_language == "DFLT" then
+      return aux.provides_script(font_id, asked_script)
+    end
+    local script = harf.Tag.new(asked_script == "dflt" and "DFLT"
+                                                        or asked_script)
+    local language = harf.Tag.new(asked_language == "DFLT" and "dflt"
+                                                            or asked_language)
+    for _, tag in next, { GSUBtag, GPOStag } do
+      local _, script_idx = hbface:ot_layout_find_script(tag, script)
+      if hbface:ot_layout_find_language(tag, script_idx, language) then
+        return true
+      end
+    end
+    return false
+  else
+    asked_language  = stringlower(asked_language)
     local tfmdata = identifiers[font_id]
     if not tfmdata then return false end
     local features = get_features (tfmdata)
@@ -505,13 +579,10 @@ local provides_language = function (font_id, asked_script, asked_language)
                "font no %d (%s) defines no feature "
                .. "for script %s with language %s",
                font_id, fontname, asked_script, asked_language)
+    return false
   end
-  logreport ("log", 0, "aux", "no font with id %d", font_id)
-  return false
 end
 
-aux.provides_language = provides_language
-
 --[[doc--
 We strip the syntax elements from feature definitions (shouldn’t
 actually be there in the first place, but who cares ...)
@@ -547,7 +618,7 @@ accounted for in the script with tag “asked_script” in feature
 --doc]]--
 
 --- int -> string -> string -> string -> bool
-local provides_feature = function (font_id,        asked_script,
+function aux.provides_feature(font_id,        asked_script,
                                    asked_language, asked_feature)
   if not font_id        or type (font_id)        ~= "number"
   or not asked_script   or type (asked_script)   ~= "string"
@@ -561,12 +632,32 @@ local provides_feature = function (font_id,        asked_script,
     return false
   end
   asked_script    = stringlower(asked_script)
-  asked_language  = stringlower(asked_language)
   asked_feature   = lpegmatch(strip_garbage, asked_feature)
 
-  if font_id and font_id > 0 then
-    local tfmdata  = identifiers[font_id]
-    if not tfmdata then return false end
+  local tfmdata = identifiers[font_id]
+  if not tfmdata then
+    logreport ("log", 0, "aux", "no font with id %d", font_id)
+    return false
+  end
+  local hbface = get_hbface(tfmdata)
+  if hbface then
+    asked_language  = stringupper(asked_language)
+    local script = harf.Tag.new(asked_script == "dflt" and "DFLT"
+                                                        or asked_script)
+    local language = harf.Tag.new(asked_language == "DFLT" and "dflt"
+                                                            or asked_language)
+    local feature = harf.Tag.new(asked_feature)
+
+    for _, tag in next, { GSUBtag, GPOStag } do
+      local _, script_idx = hbface:ot_layout_find_script(tag, script)
+      local _, language_idx = hbface:ot_layout_find_language(tag, script_idx, language)
+      if hbface:ot_layout_find_feature(tag, script_idx, language_idx, feature) then
+        return true
+      end
+    end
+    return false
+  else
+    asked_language  = stringlower(asked_language)
     local features = get_features (tfmdata)
     if features == false then
       logreport ("log", 1, "aux", "font no %d lacks a features table", font_id)
@@ -591,18 +682,14 @@ local provides_feature = function (font_id,        asked_script,
                "font no %d (%s) does not define feature %s for script %s with language %s",
                font_id, fontname, asked_feature, asked_script, asked_language)
   end
-  logreport ("log", 0, "aux", "no font with id %d", font_id)
-  return false
 end
 
-aux.provides_feature = provides_feature
-
 -----------------------------------------------------------------------
 ---                         font dimensions
 -----------------------------------------------------------------------
 
 --- int -> string -> int
-local get_math_dimension = function (font_id, dimenname)
+local function get_math_dimension(font_id, dimenname)
   if type(font_id) == "string" then
     font_id = fontid(font_id) --- safeguard
   end
@@ -617,7 +704,7 @@ end
 aux.get_math_dimension = get_math_dimension
 
 --- int -> string -> unit
-local sprint_math_dimension = function (font_id, dimenname)
+function aux.sprint_math_dimension(font_id, dimenname)
   if type(font_id) == "string" then
     font_id = fontid(font_id)
   end
@@ -625,15 +712,10 @@ local sprint_math_dimension = function (font_id, dimenname)
   texsprint(luatexbase.catcodetables["latex-package"], dim, "sp")
 end
 
-aux.sprint_math_dimension = sprint_math_dimension
-
 -----------------------------------------------------------------------
 ---                    extra database functions
 -----------------------------------------------------------------------
 
-local namesresolve      = fontnames.resolve
-local namesscan_dir     = fontnames.scan_dir
-
 --[====[-- TODO -> port this to new db model
 
 --- local directories -------------------------------------------------
@@ -642,7 +724,7 @@ local namesscan_dir     = fontnames.scan_dir
 --- https://github.com/lualatex/luaotfload/pull/61#issuecomment-17776975
 
 --- string -> (int * int)
-local scan_external_dir = function (dir)
+local function scan_external_dir (dir)
   local old_names, new_names = names.data()
   if not old_names then
     old_names = load_names()
@@ -662,7 +744,7 @@ aux.scan_external_dir = scan_external_dir
 
 --]====]--
 
-aux.scan_external_dir = function ()
+function aux.scan_external_dir()
   print "ERROR: scan_external_dir() is not implemented"
 end
 
@@ -670,14 +752,12 @@ end
 
 --- https://github.com/lualatex/luaotfload/issues/74
 --- string -> (string * int)
-local resolve_fontname = function (name)
-  local foundname, subfont, success = namesresolve(nil, nil, {
-          name          = name,
-          lookup        = "name",
-          optsize       = 0,
-          specification = "name:" .. name,
-  })
-  if success then
+local function resolve_fontname(name)
+  local foundname, subfont = luaotfload.resolvers.name {
+    name          = name,
+    specification = "name:" .. name,
+  }
+  if foundname then
     return foundname, subfont
   end
   return false, false
@@ -686,24 +766,16 @@ end
 aux.resolve_fontname = resolve_fontname
 
 --- string list -> (string * int)
-local resolve_fontlist
-resolve_fontlist = function (names, n)
-  if not n then
-    return resolve_fontlist(names, 1)
-  end
-  local this = names[n]
-  if this then
+function aux.resolve_fontlist(names)
+  for n = 1, #names do
     local foundname, subfont = resolve_fontname(this)
     if foundname then
       return foundname, subfont
     end
-    return resolve_fontlist(names, n+1)
   end
   return false, false
 end
 
-aux.resolve_fontlist = resolve_fontlist
-
 --- index access ------------------------------------------------------
 
 --- Based on a discussion on the Luatex mailing list:
@@ -722,7 +794,7 @@ aux.resolve_fontlist = resolve_fontlist
 local load_names        = fontnames.load
 local access_font_index = fontnames.access_font_index
 
-local read_font_index = function ()
+function aux.read_font_index()
   return load_names (true) or { }
 end
 
@@ -735,29 +807,18 @@ end
 
 --doc]]--
 
-local font_index = function () return access_font_index () end
+function aux.font_index() return access_font_index () end
 
-aux.read_font_index = read_font_index
-aux.font_index      = font_index
 
 --- loaded fonts ------------------------------------------------------
 
 --- just a proof of concept
 
 --- fontobj -> string list -> (string list) list
-local get_font_data get_font_data = function (tfmdata, keys, acc, n)
-  if not acc then
-    return get_font_data(tfmdata, keys, {}, 1)
-  end
-  local key = keys[n]
-  if key then
-    local val = tfmdata[key]
-    if val then
-      acc[#acc+1] = val
-    else
-      acc[#acc+1] = false
-    end
-    return get_font_data(tfmdata, keys, acc, n+1)
+local function get_font_data (tfmdata, keys)
+  local acc = {}
+  for n = 1, #keys do
+    acc[n] = tfmdata[keys[n]] or false
   end
   return acc
 end
@@ -776,27 +837,19 @@ end
 local default_keys = { "fullname" }
 
 --- string list -> (int * string list) list
-local get_loaded_fonts get_loaded_fonts = function (keys, acc, lastid)
-  if not acc then
-    if not keys then
-      keys = default_keys
-    end
-    return get_loaded_fonts(keys, {}, lastid)
-  end
-  local id, tfmdata = next(identifiers, lastid)
-  if id then
+function aux.get_loaded_fonts (keys)
+  keys = keys or default_keys
+  local acc = {}
+  for id, tfmdata in pairs(identifiers) do
     local data = get_font_data(tfmdata, keys)
     acc[#acc+1] = { id, data }
-    return get_loaded_fonts (keys, acc, id)
   end
   return acc
 end
 
-aux.get_loaded_fonts = get_loaded_fonts
-
 --- Raw access to the font.* namespace is unsafe so no documentation on
 --- this one.
-local get_raw_fonts = function ( )
+function aux.get_raw_fonts ()
   local res = { }
   for i, v in font.each() do
     if v.filename then
@@ -806,8 +859,6 @@ local get_raw_fonts = function ( )
   return res
 end
 
-aux.get_raw_fonts = get_raw_fonts
-
 -----------------------------------------------------------------------
 ---                         font parameters
 -----------------------------------------------------------------------
@@ -841,7 +892,7 @@ end})
 --doc]]--
 
 --- int -> (number | false)
-local get_quad = function (font_id)
+function aux.get_quad(font_id)
   local quad = quads[font_id]
   if quad then
     return quad
@@ -855,17 +906,61 @@ local get_quad = function (font_id)
   return false
 end
 
-aux.get_quad = get_quad
-
+-----------------------------------------------------------------------
+---                         Script/language fixup
+-----------------------------------------------------------------------
+local otftables = fonts.constructors.handlers.otf.tables
+local function setscript(tfmdata, value)
+  if value then
+    local cleanvalue = string.lower(value)
+    local scripts  = otftables and otftables.scripts
+    local properties = tfmdata.properties
+    if not scripts then
+      properties.script = cleanvalue
+    elseif scripts[value] then
+      properties.script = cleanvalue
+    else
+      properties.script = "dflt"
+    end
+  end
+  local resources = tfmdata.resources
+  local features = resources and resources.features
+  if features then
+    local properties = tfmdata.properties
+    local script, language = properties.script, properties.language
+    local script_found, language_found = false, false
+    for _, data in next, features do for _, feature_data in next, data do
+      local scr = feature_data[script]
+      if scr then
+        script_found = true
+        if scr[language] then
+          language_found = true
+          goto double_break
+        end
+      end
+    end end
+    ::double_break::
+    if not script_found then properties.script = "dflt" end
+    if not language_found then properties.language = "dflt" end
+  end
+end
+fonts.constructors.features.otf.register {
+  name         = "script",
+  initializers = {
+    base = setscript,
+    node = setscript,
+    plug = setscript,
+  },
+}
 
 -----------------------------------------------------------------------
 ---                         initialization
 -----------------------------------------------------------------------
 
-local inject_callbacks = function (lst)
+local function inject_callbacks (lst)
   if not lst and next (lst) then return false end
 
-  local inject = function (def)
+  local function inject (def)
     local cb, fn, id = unpack (def)
     cb = tostring (cb)
     id = tostring (id)
@@ -886,14 +981,10 @@ local inject_callbacks = function (lst)
   return ret
 end
 
-return {
-  init = function ()
-    local ret = true
-    luaotfload.aux = aux
-    ret = inject_callbacks (luaotfload_callbacks)
-    return ret
-  end
-}
+return function ()
+  luaotfload.aux = aux
+  return inject_callbacks (luaotfload_callbacks)
+end
 
 -- vim:tw=79:sw=2:ts=8:et
 
