@@ -3,19 +3,15 @@
 --  DESCRIPTION:  part of luaotfload / embolden
 -----------------------------------------------------------------------
 
-local ProvidesLuaModule = { 
+assert(luaotfload_module, "This is a part of luaotfload and should not be loaded independently") { 
     name          = "luaotfload-embolden",
-    version       = "3.15",       --TAGVERSION
-    date          = "2020-09-02", --TAGDATE
+    version       = "3.16",       --TAGVERSION
+    date          = "2020-12-31", --TAGDATE
     description   = "luaotfload submodule / embolden",
     license       = "GPL v2.0",
     author        = "Marcel Krüger"
 }
 
-if luatexbase and luatexbase.provides_module then
-  luatexbase.provides_module (ProvidesLuaModule)
-end  
-
 local otffeatures        = fonts.constructors.newfeatures "otf"
 
 local function enableembolden(tfmdata, _, embolden)
