@@ -9,19 +9,14 @@
 --- The bare fontloader uses a set of simplistic file name resolvers
 --- that must be overloaded by the user (i. e. us).
 
-local ProvidesLuaModule = { 
+assert(luaotfload_module, "This is a part of luaotfload and should not be loaded independently") { 
     name          = "luaotfload-resolvers",
-    version       = "2.97",       --TAGVERSION
-    date          = "2019-05-18", --TAGDATE
+    version       = "3.16",       --TAGVERSION
+    date          = "2020-12-31", --TAGDATE
     description   = "luaotfload submodule / resolvers",
     license       = "GPL v2.0"
 }
 
-if luatexbase and luatexbase.provides_module then
-  luatexbase.provides_module (ProvidesLuaModule)
-end  
-
-
 if not lualibs    then error "this module requires Luaotfload" end
 if not luaotfload then error "this module requires Luaotfload" end
 
@@ -114,11 +109,10 @@ local function resolve_name (specification)
     end
     local resolved, subfont = resolver (specification)
     if resolved then
-        logreport ("log", 1, "resolve", "name lookup %q -> \"%s%s\"",
-                   specification.name, resolved,
-                   subfont and stringformat ("(%d)", subfont) or "")
-        specification.sub = subfont
-        return resolved
+        logreport ("log", 1, "resolve", "name lookup %q -> %q",
+                   specification.name, resolved ..
+                   (subfont and stringformat ("(%d)", subfont) or ""))
+        return resolved, tonumber(subfont)
     end
     return resolve_file (specification)
 end
@@ -149,18 +143,19 @@ end
 
 local tex_formats = { "tfm", "ofm" }
 
-local resolve_tex_format = function (specification)
+local resolvers_findfile = luaotfload.fontloader.resolvers.findfile
+local function resolve_tex_format (specification)
     local name = specification.name
     for i=1, #tex_formats do
         local format = tex_formats [i]
-        local resolved = resolvers.findfile(name, format)
+        local resolved = resolvers_findfile(name, format)
         if resolved then
             return resolved, format
         end
     end
 end
 
-local resolve_path_if_exists = function (specification)
+local function resolve_path_if_exists (specification)
     local spec = specification.specification
     local exists = lfsisfile (spec)
     if exists then
@@ -179,7 +174,7 @@ end
     Custom file resolvers via callback.
 --doc]]--
 
-local resolve_my = function (specification)
+local function resolve_my (specification)
     return luatexbase.call_callback ("luaotfload.resolve_font", specification)
 end
 
@@ -191,7 +186,7 @@ local resolve_methods = {
     my   = resolve_my,
 }
 
-local resolve_sequence = function (seq, specification)
+local function resolve_sequence (seq, specification)
     for i = 1, #seq do
         local id  = seq [i]
         local mth = resolve_methods [id]
@@ -233,7 +228,7 @@ local function resolve_kpse (specification)
     local name       = specification.name
     local suffix     = stringlower (filesuffix (name))
     if suffix and fonts.formats[suffix] then
-        local resolved = resolvers.findfile(name, suffix)
+        local resolved = resolvers_findfile(name, suffix)
         if resolved then return resolved end
     end
     for t, format in next, fonts.formats do --- brute force
@@ -242,13 +237,23 @@ local function resolve_kpse (specification)
     end
 end
 
+local lookup_subfont_index = fonts.names.lookup_subfont_index
 local function wrap_resolver(resolver)
     return function (specification)
-        local filename, forced = resolver(specification)
+        local filename, sub, forced = resolver(specification)
+        if type(sub) ~= "number" then
+            forced = sub
+            sub = nil
+        end
         if filename then
             specification.resolved = filename
             specification.filename = filename
             specification.name = filename
+            specification.sub = sub or specification.sub
+            if type(specification.sub) == "string" then
+                specification.sub =
+                    lookup_subfont_index(filename, specification.sub)
+            end
             specification.forced = specification.forced or forced
             if not specification.forced then
                 local suffix = stringlower (filesuffix (filename))
@@ -275,28 +280,25 @@ local resolvers = table.merge(luaotfload.resolvers, {
 })
 luaotfload.resolvers = resolvers
 
-return {
-    init = function ( )
-        if luatexbase and luatexbase.create_callback then
-            luatexbase.create_callback ("luaotfload.resolve_font",
-                                        "exclusive", function () end)
-        end
-        logreport ("log", 5, "resolvers", "installing font resolvers", name)
-        local request_resolvers = fonts.definers.resolvers
-        for k, _ in pairs(resolvers) do
-            request_resolvers[k] = nil
-        end
-        setmetatable(request_resolvers, {__index = function(t, n)
-            if not resolvers[n] then return end
-            local wrapped = wrap_resolver(resolvers[n])
-            t[n] = wrapped
-            return wrapped
-        end})
-        fonts.formats.ofm      = "type1"
-        fonts.encodings        = fonts.encodings       or { }
-        fonts.encodings.known  = fonts.encodings.known or { }
-        return true
-    end, --- [.init]
-}
-
+return function()
+    if luatexbase and luatexbase.create_callback then
+        luatexbase.create_callback ("luaotfload.resolve_font",
+                                    "exclusive", function () end)
+    end
+    logreport ("log", 5, "resolvers", "installing font resolvers", name)
+    local request_resolvers = fonts.definers.resolvers
+    for k, _ in pairs(request_resolvers) do
+        request_resolvers[k] = nil
+    end
+    setmetatable(request_resolvers, {__index = function(t, n)
+        if not resolvers[n] then return end
+        local wrapped = wrap_resolver(resolvers[n])
+        t[n] = wrapped
+        return wrapped
+    end})
+    fonts.formats.ofm      = "type1"
+    fonts.encodings        = fonts.encodings       or { }
+    fonts.encodings.known  = fonts.encodings.known or { }
+    return true
+end
 --- vim:ft=lua:ts=8:sw=4:et:tw=79
